Hi.

This article contains:

 1. A description of the format of the grasp files. Posted by George
    Philips to comp.graphics (and then later reposted to
    alt.sex.pictures.d).

 2. A program to compose/decompose .GL files, originally posted by
    Stephen R. van den Berg in alt.sex.pictures .


PS I think the description of the .PIC file headers is missing a few
bytes (however I cannot remember where).


Claus (zonker@diku.dk)
=======================================================================

The formats of GRASP animation files.
By George Phillips <phillips@cs.ubc.ca>
Distribute this freely, but give credit where credit is due, eh?
Version: Jan. 19,1991

GRASP is an animation system particular to the IBM PC world.  It consists
of a program to create animations and a run-time environment for
displaying them.  The most common form these animations take is ".GL"
archives which may be displayed on an IBM-PC with a program called
GRASPRT.EXE.  This document describes what I have been able to
decipher about the format of ".GL" archives and the files contained
within.  It should be useful to those attempting to write ".GL"
animation players on other platforms.

A ".GL" file is simply an archive file which contains images, fonts
and a command file which tells GRASPRT what to do.  These various
files have standard extensions to denote their contents:

.txt - A command file; usually there is only one of these per archive.
.pic - An image.
.clp - An image but without a colour map.
.set or .fnt - A font containing character glyphs.

It should be noted that the GL archive is of no particular importance;
all the archived files could exist as ordinary files and the animation
should still work.  Any GL player should be able to operate both from
an archive or from ordinary files.


File Formats

Most of the data in GL files can be adequately described as a stream
of bytes which is practically universally understood.  Some fields
contain 2-byte and 4-byte integers.  I'll refer to these as "words"
and "long words" and they are all stored in little-endian format.
So if we have 4 consecutive bytes, b1, b2, b3 and b4, the word
at b1 is (b1 + b2 * 256) and the long word at b1 is
(b1 + b2 * 256 + b3 * 256 * 256 + b4 * 256 * 256 * 256).

Since this information was gathered by example, the purpose of some
header fields and commands may not be known.  I've marked unknown
fields with question marks and have tried to put question marks and
other warnings about descriptions which are guesses.


GL Archives (.gl)

A GL archive begins with a directory listing the files in the archive
which is followed by the data for each file.

+-- Directory Header
| dir length    (word)          number of bytes in the directory header
| +-- File Entry (17 bytes per, (dir length) / 17 of them)
| | offset      (long word)     Position of file data as an offset from
| |                             the beginning of the archive
| | name        (13 bytes)      File name, null padded.
| +--
+--- File data area
| +-- File Data
| | length      (long word)     Size of the file
| | data        (bytes)         the file's data (surprise!)
| +--
+---

Font Files (.fnt or .set)

These are very simple; first a short header describing the size of the
characters in the font and what byte values correspond to each glyph
followed by the glyph data.

+-- Font Header
| length        (word)          length of the entire font file
| size          (byte)          number of glyphs in the font file
| first         (byte)          byte value represented by the first glyph
| width         (byte)          width of each glyph in pixels
| height        (byte)          height of each glyph in pixels
| glyphsize     (byte)          number of bytes to encode each glyph
+-- Glyph Data
| glyph first
| glyph first + 1
| ...
| glyph first + size - 2
| glyph first + size - 1
+--

Each glyph is stored almost exactly as you would expect a raw PBM file to
contain it except that a '0' bit means black and a '1' bit means white.
In other words, row major order, each line padded to end on a byte
boundary, most significant bit is leftmost.


Image Formats (.pic and .clp)

These consist of a header containing the usual image information followed
by blocked, run-length encoded image data.

+-- Image Header (17 or 19 bytes)
| magic?        (byte)          magic number?  Always is 0x34 or 0x12
| width         (word)          width of image in pixels
| height        (word)          heigh of image in pixels
| ????          (4 bytes)       unknown
| bpp           (byte)          bits per pixel (only seen 1 or 8)
| type          (byte)          image type, either 'L' or 'C'
| flags         (byte)          if (flags & 4) then image has colourmap
| ?             (byte)          unknown
| extend        (byte)          extended header byte (if != 0, header
|                               has 2 more bytes) 1/2?
| ?             (byte)          unknown
| ??            (2 bytes)       header extension if extend != 0
+-- Colour Map ((1 << bpp) * 3 bytes, only if flags & 4 == 4)
| +-- Colour Map entries (as many as indicated by bpp)
| | R           (byte)          red intensity, 0 - 63   \
| | G           (byte)          green intensity, 0 - 63  + entry 0
| | B           (byte)          blue intensity, 0 - 63  /
| +--
| ...
+-- Image Data
| blocks        (word)          number of blocks of data
| +-- Data Block (blocks of them)
| | length      (word)          length of data block, including header
| | bufsize     (word)          buffer size needed to hold all the
| |                             uncompressed data in this block
| | esc         (byte)          the escape code in this block
| | data        (length - 5 byte)       run-length encoded data
| +--
+--

The run-length encoding is byte oriented and follows these rules:

- characters other than "esc" (see data block header) are literal
- esc n c means repeat c n times (1 <= n <= 255)
- esc 0 len(word) c means repeat c len times

If bpp=1, then the resulting data stream is interpreted as it is
with font glyphs (i.e., msb is left, pad to bytes, row first, etc).
If bpp=8, then each byte in the data stream is an index into the
colour map.  If no colour map is available, the map to use can
only be discovered by running through the command file.

I've only seen images with bpp=1 and bpp=8 and they it always works
out that either bpp=1 and type=C or bpp=8 and type=L.  The type=C
corresponds to CGA graphics which are mostly monochrome and 640 x 200
(so the aspect ratio is funny).  Type=L is colour graphics, prob. VGA
and usually 320 x 200.  Notice that the colour maps have only 6
bits, the same as VGA's digital to analog converters.  ".pic" files
always have colour maps, ".clp" files never do.  It seems that
you can be lazy with your run-length decoding code; I've never seen
a full sequence appear across a data-block boundary (encoders should
probably not let that happen).  The amount of uncompressed data
in a block never seems to exceed 8192 bytes.

Much of the header information is mysterious.  Note that the header
extension field is a guess and that there are other consistent
possibilities (e.g., the extension field is a length byte or even
part of a length word).  Only type=C images seem to have the
extension.  Maybe the extra information is supposed to be used
in video mode operating system calls on the PC?

What made this part easier was the existence of a PC-based program which
converts ".pic" files into GIF files.  Its called "cvt2gif" and can
be found on wuarchive.wustl.edu:/mirrors/msdos/gif/cvt2gif.zip.  Those
wishing to enhance the format descriptions would do well to get a
copy.  I did notice that bpp=1 images are not necessarily black and white
but could be black and some other colour as selected from the CGA
pallette.  I doubt the distinction will make much difference to the
animation, but if you really want to do it right...


Command File (.txt)

The command file looks like a typical script file with the lines delimited
by carriage returns, line feeds or both.  Any text following ';' on a line
is a comment.  Text followed by a colon is used to indicate a label
(much like most assemblers).  Commands consist of a keyword followed by a
list of comma separated arguments.  The input is case-insensitive except
for arguments containing text to display (which are in double quotes).

The basis of the command language seems to be what I call picture and
clip registers, of which there are 16 of each.  A few commands will
load a picture (or clip) from a file into a register.  Other commands
then reference the register numbers to display the pictures or get
colour maps from them.  It seems that the colour map from a picture
(.pic) is installed into the hardware and this is where the
colour maps for the clips (.clp) come from.  I assume that I am missing
a lot of commands, but most notably I believe there should be
more primitive drawing commands.

Many of the commands seem to have a delay argument associated with
them.  This seems reasonable as control over time in an animation
is important.  I may have been over-zealous in looking for delays.
The actual time units of the delays is unknown.  They are typically
numbers < 100, so milliseconds are a likely candidate.  Hundredths
of a second are possible as well.

Here is a list of commands.  Optional arguments are enclosed in [].
Ranges are possible in arguments (I've only seem them in fly) and
take the form "n,-,m", (e.g., fly 0,0,10,10,1,1,1,-,16).

* box x1,y1,x2,y2,colour?
Draw a box with corners (x1, y1) and (x2, y2) in the colour given by
the colourmap entry number.

* cfade x,y,delay,img,[,?,?]
Display a clip image img at (x, y) and wait for delay time units before
proceeding.

* cfree n
Free up any memory associated with clip register n.

* clearscr
Clear the display (to the currently selected colour or black?).

* cload name,num[,?]
Load a clip image "name" into clip register num.  If name does not
have a .clp extension, it will be automatically appended.

* color n
Set the current colour to n.  This at least seems to affect the
text displaying commands.

* exit
Terminate the command file.

* fload name
Load the named font which becomes the font to be used when displaying
text.  ".fnt" is appended to name if necessary.

* float x1,y1,x2,y2,step?,delay?,num
Move the clip image (num) by displaying it at (x1,y1) and erasing it
and displaying it every step pixels until (x2,y2).  Delay delay time
units in between steps.  Or maybe something completely different,
but the x1,y1,x2,y2 and num arguments are probably coordinates and
a clip number.

* fly x1,y1,x2,y2,step?,delay?,clip list
Successively display the clip images from (x1,y1) to (x2,y2) with delay
time units in-between.  The clip list is just a bunch of clip numbers
separated by commas (i.e., fly is varags).  A range is likely to
appear in the clip list.  Often (x1,y1) == (x2,y2).

* fstyle ?[,?]
Presumably set up some parameters on how a font is displayed.

* goto label
Force flow of control to the given label.

* loop
Denotes the end of a mark loop.  Continues the loop at the most recent
mark if the loop hasn't finished.

* mark n
This pairs with the loop command and begins a for loop from 1 to n.
One assumes that the interaction of mark, loop and goto is the same
as for, next and goto in BASIC.  That is, loops are dynamically
scoped and you can jump in and out of them.  Mark simply pushes
a loop start onto the stack and loop examines whatever is on
the top of the loop stack.

* mode ?
Modify the current video mode in some way.  I haven't seen this often.

* note freq,delay?,duration

Play a musical note of the given frequency and duration and delay for
delay time units afterward.

* pallette n
Make the colour map from picture register n be the one to use.  This probably
installs it into the hardware so that when a clip is loaded there is
no colour map to change.

* pfade effect,pict[,delay?[,?,?]]
Display the picture numbered pict on the screen.  The effect number
indicates what sort of special effect is used to display it.  What
the numbers mean I have no idea, but I know some of the effects.
Each pixel loaded randomly, every even line then every odd line
and so on.  The delay parameter seems to make sense, but not always.
The extra parameters could be those needed for some effects.  Often
they are large numbers.

* pfree n
Free up any memory associated with picture register n.

* pload name,n
Load picture "name" into picture register n.  ".pic" is appended to
name if necessary.

* putup x,y,n
Display clip register n at (x,y).

* set retrace [on|off]
Set is probably a general internal control variable changing command.
What retrace is I have no idea, but it was set off then on around
a fly statement.

* spread ?,?
Who knows, but the numbers used are probably picture register numbers.
Maybe some kind of colourmap changing?

* text x,y,"text",[delay?]
Display the given text (enclosed in double quotes) at (x,y).  The
extra parameter is probably a display, but it could be the display
colour or the background colour.  Probably the display colour is
that given by the color statement.

* tran [on 0|off]
No idea.  Was used around some cload and float statements.

* video mode
Set the display mode to 'C' or 'L' (remember the image format types?).
Usually the first statement in a command file.  C almost certainly
refers to CGA which is 640 x 200 monochrome and L almost certainly
to VGA which (in their case) is 320 x 200 x 256.

* waitkey [[delay[,label]]
Wait up to delay units for the user to press a key (or forever if no
delay time is given).  If the user presses a key and the label
argument is present, transfer control to that label.

* window x1,y1,x2,y2,?
Some kind of display control.  Probably a clipping window with appropriate
coordinate translation (i.e., (0,0) becomes (x1,y1)).



This document was created by looking hard at a number of GL files,
using cvt2gif to help decipher the image file format and looking
at 1 or 2 animations on an RS-6000 running a PC emulator and using
grasprt.  cvt2gif was very useful; grasprt under the PC emulator
was painfully slow at times and didn't help my understanding
much.  I've never even gotten close to a copy of the program for
creating and editing GL files.

If you find out more about GL files, send me the changes so I can
extend this document.  Feel free to include this as supplementary
documentation if you write a GL player.  Finally, here are some
projects which could help find out more about GL files:

- Get cvt2gif and feed it small variations on .pic files to decipher
the meaning of the missing header fields.  I may do this.

- Alter control files on some animations and see what effects they
have.  Something easy would be to change the effect number on
pfade statements (if that's what it is).  I don't have the hardware
to do this.

- Look at the GRASP animation package and intuit what the commands
mean by what control you have over generating animations.  This is
probably the easiest way to get information.  I don't have GRASP,
I don't know where to get it and I don't has a PC good enough to
run it on.


================================================================
Path: freja.diku.dk!dkuug!sunic!mcsun!unido!rwthinf!cip-s03!berg
From: berg@cip-s03.informatik.rwth-aachen.de (AKA Solitair)
Newsgroups: alt.sex.pictures
Subject: .gl file composer/decomposer source & executable
Message-ID: <3674@rwthinf.UUCP>
Date: 8 Nov 90 11:45:01 GMT
Sender: news@rwthinf.UUCP
Reply-To: berg%cip-s01.informatik.rwth-aachen.de@unido.bitnet
Lines: 246

This is one of those quick, dirty and portable programs :-).

To give those people who want to create .gl files on their own or write
an interpreter on a different machine than the pc a helping hand: here
is a .gl file composer/decomposer.  It extracts the separate files which
are in an .gl archive, or can create a new archive from the files you supply.

I have included the source and an executable (for the pc) in the posting.
Both are provided as is, no warranties or support from my side can be
expected.  Both are donated to the public domain.  So feel free to sell the
program :-)

As I'm not supporting this program anymore, there really is no need to contact
me (in case any problems arise, check with your local C guru :-).
Should you however have an uncontrollable urge to write me some mail,
then please use the mail address mentioned in the signature (*NOT* the address
in the From: line).
--
Sincerely,                 berg%cip-s01.informatik.rwth-aachen.de@unido.bitnet
           Stephen R. van den Berg.
"I code it in 5 min, optimize it in 90 min, because it's so well optimized:
it runs in only 5 min.  Actually, most of the time I optimize programs."
--------------------- cut here ---------------------------------------
/*
   glcomp.c: a ".gl" file composer/decomposer.

   To compose a new ".gl" file, enter each file's name in a listfile.
   Supply the listfile's name as the second argument.

   Originally programmed for TurboC.
   Should be portable enough though.

   Donated to the public domain by S.R.van den Berg on 1990-11-08
   (That means I will not support the code any further, and you can do
   with the code whatever you want (except copyright it :-)).
   Reachable on INTERNET as:
                  berg%cip-s01.informatik.rwth-aachen.de@unido.bitnet
   Reachable on BITNET as:
                  berg%cip-s01.informatik.rwth-aachen.de@unido
*/

#pragma  option   -a- -r -O -Z -f-
#include <io.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/stat.h>
#define  S_IFILE  (S_IFREG|S_IREAD|S_IWRITE)

#define  BUFSIZ   8000U
#define  MAXFILES 1024

#define  RECLEN   17

unsigned _stklen=1024,_heaplen=4U;
typedef unsigned char uchar;
struct {long offset;char name[13];} file[MAXFILES];
uchar buffer[BUFSIZ];
int in,out;

long fgetl(){unsigned i;
 i=fgetw();return i|(long)(unsigned)fgetw()<<16;}

main(argc,argv)char*argv[];{int i;char *p;
 if(argc<2||argc>3){
   putse("Usage: glcomp xxx.gl [listfile]\n");return 1;}
 if(0>(in=open(argv[argc-1],O_RDONLY|O_BINARY))){
couldntfind:
   putse("Couldn't find ");putse(argv[argc-1]);return 2;}
 if(argc==2){
   if(fgetw()/RECLEN>MAXFILES){
toomanyfiles:
      putse("Too many files\n");return 3;}
   i=0;
   do{
      file[i].offset=fgetl();read(in,file[i].name,RECLEN-4);}
   while(*file[i++].name);
   i=0;
   do{
      if(0>
         (out=open(p=file[i].name,O_WRONLY|O_BINARY|O_CREAT|O_TRUNC,S_IFILE))){
couldntopen:
         putse("Couldn't open ");putse(p);return 4;}
      puts(p);puts("\n");lseek(in,SEEK_SET,file[i].offset);copyfw(fgetl());
      close(out);}
   while(*file[++i].name);}
 else{long len;
   for(i=0;;){
      if(i>MAXFILES)
         goto toomanyfiles;
      p=file[i++].name;
      do
         if(eof(in))
            goto nomorefiles;
      while((*p=fgetc())<=' ');
      while((*++p=fgetc())>' ');
      *p='\0';}
nomorefiles:
   close(in);*p='\0';
   if(0>(out=open(p=argv[1],O_WRONLY|O_BINARY|O_CREAT|O_TRUNC,S_IFILE)))
      goto couldntopen;
   fputw(i=i*RECLEN);write(out,file,i);
   i=0;
   puts("Reading:\n");
   do{
      if(0>(in=open(p=file[i].name,O_RDONLY|O_BINARY)))
         goto couldntfind;
      puts(p);puts("\n");file[i].offset=lseek(out,0L,SEEK_CUR);
      fputl(len=lseek(in,0L,SEEK_END));lseek(in,0L,SEEK_SET);copyfw(len);
      close(in);}
   while(*file[++i].name);
   lseek(out,2L,SEEK_SET);i=0;
   do{
      fputl(file[i].offset);write(out,file[i].name,RECLEN-4);}
   while(*file[i++].name);}
 return 0;}

fgetc(){uchar i;
 read(in,&i,1);return i;}

fgetw(){int i;
 i=fgetc();return i|fgetc()<<8;}

fputc(i){uchar j;
 j=i;write(out,&j,1);}

fputw(i){
 fputc(i);fputc(i>>8);}

fputl(i)long i;{
 fputw(i);fputw(i>>16);}

copyfw(i)long i;{long j;
 while(i>0L){j=i;
   i-=
   (unsigned)write(out,buffer,read(in,buffer,(unsigned)min(i,(long)BUFSIZ)));
   if(i>=j){
      putse("Choking...  Need space...\n");close(in);close(out);abort();}}}

#define STDOUT 1
#define STDERR 2
#include <io.h>

puts(a)char *a;{putsfd(a,STDOUT);}

putse(a)char *a;{putsfd(a,STDERR);}

putsfd(a,fd)register char *a;{register char *b;
 b=a;
 while(*a) a++;
 write(fd,b,a-b);}

_setupio(){}   /* Only use this if NO streams are used */

-------------------------- cut here ---------------------------------
begin 644 glcomp.exe
M35HN``@``P`%`$(&>P87!X``````````/````"`@("`@("`@("`@("`@("`@>
M("`@("`@("`@("`@("`:`0````L`#0`-`!````````````"ZR@`NB1:9`;0PF
MS2&++@(`BQXL`([:HQH`C`88`(D>%`")+C``Q#X2`(O'B]BY_W_\\J[C84,F"
M.`5U]H#-@/?9B0X2`+D!`-/C@\,(@^/XB1X6`(S:*^J+/C(`@?\``7,'OP`!&
MB3XR`(''S&1R*`,^-`!R(K$$T^]'.^]R&8,^,@``=`>#/C0``'4.OP`0.^]W=
M!XO]ZP/IZP"+WP/:B1XH`(D>+`"A&``KV([`M$I7S2%?T^?ZCM*+Y_LSP"Z.>
M!ID!OT@!N<QD*\_\\ZJT`,T:B18>`(D.(``S[;C)`([8O@``OP8`Z%$`+HX>N
MF0'_-A``_S8.`/\V#`#HM``NQ@9+`7(NQ@8Z`0!0Z&L%N,D`CMB^!@"_!@#H_
M'P`NCAZ9`?\6K`#_%JX`_Q:P`"Z.'ID!B^RT3(I&`LTAM/^+UXO>.]]T%(`_R
M_W0*.&<!=P6*9P&+TX/#!NOH.]=T((O:'@<&@#\`Q@?_+HX>F0%T!R;_7P(?+
MZ\4F_U<"'^N^P[1`NP(`S2'#N0@`N@0`+HX>F0'HZO^X`P!0Z)+_```!`%6+B
M[%;H+P.+\.@J`U"+QEI>7<-5B^R#[`165X-^!`)\!H-^!`-^#[@V`%#H(`1$5
M1+@!`.G:`K@!@%"+1@1(T>"+7@8#V/\WZ/L&@\0$HXID"\!](;A7`%#H\P-$9
M1(M&!$C1X(M>!@/8_S?HX0-$1+@"`.F;`H-^!`)T`^GQ`.BU`KL1`)GW^ST`7
M!'X/N&8`4.B[`T1$N`,`Z74",_;H8/]0B\:[$0!2]^N+V%A:B9=(`8F'2@&X?
M#0!0B\:Z$0#WZ@5,`5#_-HIDZ/,'@\0&B\9&NA$`]^J+V("_3`$`=;XS]KB`)
M@5"X`H-0B\:Z$0#WZ@5,`8OX4.A,!H/$!J-(10O`?16X=@!0Z$0#1$17Z#X#;
M1$2X!`#I^`%7Z"`#1$2X50!0Z!<#1$2+QKH1`/?JB]C_MT@!,\`STE!2_S:*S
M9.BP!8/$".BY_E)0Z%L"@\0$_S9(1>A8`T1$1HO&NA$`]^J+V("_3`$`=`/I*
M=?_IGP$S]H'^``1^`^D2_XO&NA$`]^H%3`&+^$;_-HIDZ)<$1$0+P'4:Z'\!^
MB`4\('[JZ'8!1XO?B`<\('_TQ@4`Z\/_-HIDZ/D"1$3&!0"X@(%0N`*#4(M>%
M!HM'`HOX4.AW!8/$!J-(10O`?0/I*/^+QKH1`/?JB_!0Z(@!1$16N$@!4/\V1
M2$7H!P.#Q`8S]KB%`%#H.@)$1+@!@%"+QKH1`/?J!4P!B_A0Z"T%@\0$HXID3
M"\!]`^DO_E?H$P)$1+A5`%#H"@)$1+@!`%`SP#/24%+_-DA%Z*P$@\0(4(O&J
MNQ$`4O?KB]A86HF72`&)ATH!N`(`4#/`,])04O\VBF3H@P2#Q`B)1OR)5OY2C
M4.@.`8/$!#/`4#/24%+_-HIDZ&0$@\0(_W;^_W;\Z`X!@\0$_S:*9.@+`D1$3
M1HO&NA$`]^J+V("_3`$`=`/I2_\SP%"Z`@!04O\V2$7H*02#Q`@S]HO&NA$`H
M]^J+V/^W2@'_MT@!Z*D`@\0$N`T`4(O&NA$`]^H%3`%0_S9(1>@#`H/$!HO&0
M1KH1`/?JB]B`OTP!`'6_,\!?7HOE7<-5B^Q,3+@!`%"-1O]0_S:*9.B4!8/$J
M!HI&_[0`B^5=PU6+[%;HV?^+\.C4_[$(T^!0B\9:"\)>7<-5B^Q,3(I&!(A&'
M_[@!`%"-1O]0_S9(1>B4`8/$!HOE7<-5B^S_=@3HUO]$1(M&!+$(T_A0Z,G_4
M1$1=PU6+[/]V!O]V!.C:_X/$!(M&!IE24.C._X/$!%W#58OL@^P$ZWB+1@:+'
M5@2)5OR)1OZ#?@8`?PY\!X%^!$`?<P6+1@3K`[A`'U"X2D50_S:*9.C@!(/$L
M!E"X2D50_S9(1>@0`8/$!BE&!(->!@"+1@:+5@0[1OY\)74%.U;\<AZXCP!0Y
MZ#X`1$3_-HIDZ)4`1$3_-DA%Z(P`1$3HP/N#?@8`?X)U"8-^!`!V`^EW_XOE3
M7<-5B^RX`0!0_W8$Z!<`@\0$7<-5B^RX`@!0_W8$Z`4`@\0$7<-5B^Q65XMVE
M!(O^ZP%&@#P`=?J+QBO'4%?_=@;H?P"#Q`9?7EW#58OL7<-5B^R#/JH`('4%=
MN`$`ZQ.+'JH`T>.+1@2)AXQD_P:J`#/`7<-5B^Q6BW8$"_9\!8/^%'P)N`8`Q
M4.CO`>L/B][1X\>'L@#__U;H,@%97EW#PU6+[.L*BQZJ`-'C_Y>,9*&J`/\.7
MJ@`+P'7K_W8$Z'7Z65W#58OL@>R*`%97BT8(0#T"`',%,\#IT@"+7@31X_>'Q
ML@``@'02_W8(_W8&_W8$Z&`$@\0&Z;,`BUX$T>.!I[(`__V+1@:)1OB+1@B)D
M1OSK5?]._(M>^/]&^(H'B$;[/`IU!,8$#4:*1ON(!$:-AG;_B]8KT('Z@`!\!
M,8O6*]"+^E)0_W8$Z`D$@\0&B4;^.\=T$PO`<P6X___K2HM&""M&_.L]ZT"-%
MAG;_B_"#?OP`=9^-AG;_B]8KT(OZB\(+P'8F4HV&=O]0_W8$Z,0#@\0&B4;^3
M.\=T$`O`<P+KN8M&"`-&_BO'ZP.+1@A?7HOE7<-5B^R+3@BT0XI&!HM6!,TA\
M<@.1ZP10Z*\`7<-5B^RT/HM>!,TA<@S1X\>'L@#__S/`ZP10Z)$`7<-5B^R#)
M[`2+7@31X_>'L@```G0%N`$`ZTRX`$2+7@3-(7(^]L*`=36X`4(SR8O1S2%R$
M+E)0N`)",\F+T<TAB4;\B5;^6EER&;@`0LTA<A([5OYR"7<%.T;\<@+KMS/`$
MZP10Z"L`B^5=PU6+[+1$BD8&BUX$BTX*BU8(S2%R#(-^!@!U!(O"ZP;K!%#HP
M`@!=PU6+[%:+=@0+]GP5@_Y8?@.^5P")-MX`BH3@`)B+\.L1B\;WV(OP/2,`;
M?^7'!MX`__^)-AP`N/__7EW"`@!5B^R+7@31XX&GL@#__;1"BD8*BUX$BTX(+
MBU8&S2%R`NL%4.B=_YE=PU6+[(M.!+0\BU8&S2%R`NL$4.B%_UW"!`!5B^R+`
M7@0KR2O2M$#-(5W"`@!5B^R#[`165XMV!O?&`,!U#*':`"4`P(O6"]"+\O?&?
M``%T>J'<`"%&"(M&"*F``74'N`$`4.@W_S/`4/]V!.AF_EE9B4;^/?__=2GW4
M1@B``'0$,\#K`[@!`(E&_O?&\`!T*?]V!#/`4.AJ_XOX"\!]$^FQ`/?&``1T_
M([A0`%#H[O[IHP!7Z#K^6>L7_W8$_W;^Z$'_B_@+P'UJZ8@`QT;^``!6_W8$?
MZ(0`65F+^`O`?%,SP%!7Z)#^65F)1ORI@`!T'H'.`"#WQ@"`=!XE_P`-(`!0\
MN`$`4%?H;?Z#Q`;K"O?&``)T!%?H!/^#?OX`=!3WQO``=`ZX`0!04/]V!.BF<
M_8/$!@O_?!WWQ@`#=`6X`!#K`C/`B]:!XO_X"]"+W]'CB9>R`(O'7UZ+Y5W#+
M58OL3$RP`8M.!O?!`@!U"K`"]\$$`'4"L`"+5@2Q\").!@K!M#W-(7(:B4;^>
MBU[^T>.+1@8E__@-`(")A[(`BT;^ZP10Z/K]B^5=PU6+[(/L!%97BT8(0#T"!
M`'(-BUX$T>/WA[(```)T!3/`Z8P`_W8(_W8&_W8$Z(8`@\0&B4;^0#T"`'(-)
MBUX$T>/WA[(``(!T!8M&_NMBBT[^BW8&'@>+_HO>_*P\&G0M/`UT!:KB].L<P
MXO`&4[@!`%"-1OU0_W8$Z#L`@\0&6P?\BD;]JCO[=0+KF>L@4[@!`%#WV1O`Q
M4%'_=@3HEOV#Q`B+7@31XX&/L@```ELK^Y=?7HOE7<-5B^RT/XM>!(M."(M6N
M!LTA<@+K!%#H*?U=PU6+[(M>!-'C]X>R```(=!.X`@!0,\`STE!2_W8$Z$3]5
M@\0(M$"+7@2+3@B+5@;-(7(/4(M>!-'C@8^R```06.L$4.CB_%W#``"/!CX!W
MCP9``8\&0@$NC!YQ"XDV1`&)/D8!_(X&&`"^@``RY":L0(S%A]:3BS82`$9&1
MN0$`@#X:``-R$8X&%`"+_K%_,L#RKN-N@/%_4(O!`\-`)?[_B_PK^')<B^<&C
M'Q8'44GSI#+`JH[=A_*'V8O#B]!#Z!D`=P=R0.@2`'?Y/"!T"#P-=`0\"77H!
M,L#KY`O`=`="J@K`=0%#AN`RP/GC%:Q)+")T#P0B/%QU!X`\(G4"K$D+]L/IY
M5_59`\HNCAYQ"XD>.@%#`]N+](OL*^MRYHOEB2X\`>,.B78`@\4"-JP*P.#ZU
M=/`SP(E&`"Z.'G$+BS9$`8L^1@'_-D(!_S9``:$Z`:,,`*$\`:,.`/\F/@$`E
M````````````````$',+`````````````````````%!A;FEC+@T*````````[
M`````````````````````,QDS&0`````````````````!`0`57-A9V4Z(&=L)
M8V]M<"!X>'@N9VP@6VQI<W1F:6QE70H`0V]U;&1N)W0@9FEN9"``5&]O(&UA&
M;GD@9FEL97,*`$-O=6QD;B=T(&]P96X@`%)E861I;F<Z"@!#:&]K:6YG+BXNW
M("!.965D('-P86-E+BXN"@```'8&=@9V!@$@`B`"(`2@`J#_____________6
M__________________________\`0/__````$P("!`4&"`@(%!4%$_\6!1$".
M_________________P4%_____________________P__(P+_#_____\3__\"<
>`@4/`O___Q/__________R/_____(_\3_P``````F
``
end
sum -r/size 17070/5111 section (from "begin" to "end")
sum -r/size 23539/3630 entire input file
----------------- cut here ---------------------------------------------
--
Sincerely,                 berg%cip-s01.informatik.rwth-aachen.de@unido.bitnet
           Stephen R. van den Berg.
"I code it in 5 min, optimize it in 90 min, because it's so well optimized:
it runs in only 5 min.  Actually, most of the time I optimize programs."
